#ifndef ZEGO_WHITEBOARD_MANAGER_H
#define ZEGO_WHITEBOARD_MANAGER_H

#include "zego-whiteboard-defines.h"

#ifdef __cplusplus
extern "C" {
#endif

	/**
	 * 设置log路径
	 *
	 * @param log_path log保存路径
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_set_log_folder(const char* log_path);

	/**
	 * 设置SDK缓存路径
	 *
	 * 当使用文件白板时，此方法必须设置
	 *
	 * @param cache_path SDK缓存路径
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_set_cache_folder(const char* cache_path);

	/**
	 * 获取SDK版本号
	 *
	 * @return SDK版本号
	 */
	ZEGOAPI const char* ZEGOCALL zego_whiteboard_get_version();

	/**
	 * 注册错误通知
	 *
	 * @param fn 回调函数地址
	 * @param user_context 用户自定义数据
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_reg_error_notify(zego_whiteboard_error_notify fn, void* user_context);

	/**
	 * 设置是否使用测试环境
	 *
	 * @param is_test 是否使用测试环境
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_set_test_env(bool isTese);

	/**
	 * 设置用户自定义属性
	 *
	 * @return 成功或失败
	 */
	ZEGOAPI bool ZEGOCALL zego_whiteboard_set_custom_config(const char* key, const char* value);

	/**
	 * 初始化SDK
	 *
	 * 在初始化房间之后，在登录房间之前调用
	 *
	 * @param app_sign byte数组 例如{0x7f, 0xa0, 0xba,....}
	 * @param sign_len app_sign数组长度
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_initialize(unsigned int app_id, const char* app_sign, unsigned int sign_len);

	/**
	 * 注册SDK初始化回调
	 *
	 * @param fn 回调函数地址
	 * @param user_context 用户自定义数据
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_reg_initialize_callback(zego_whiteboard_initialize_callback fn, void* user_context);

	/**
	 * 反初始化SDK
	 *
	 * 在退出房间之后，反初始化房间之前调用
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_uninitialize();

	/**
	 * 注册SDK反初始化回调
	 *
	 * @param fn 回调函数地址
	 * @param user_context 用户自定义数据
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_reg_uninitialize_callback(zego_whiteboard_uninitialize_callback fn, void* user_context);

	/**
	 * 获取当前房间内所有白板
	 *
	 * 此操作会请求后台拉取房间内的白板，一般在进入房间后立即拉取一次
	 *
	 * @return 请求序号
	 */
	ZEGOAPI ZegoSeq ZEGOCALL zego_whiteboard_get_whiteboard_list();

	/**
	 * 注册获取白板列表回调
	 *
	 * @param fn 回调函数地址
	 * @param user_context 用户自定义数据
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_reg_get_whiteboard_list_callback(zego_whiteboard_get_whiteboard_list_callback fn, void* user_context);

	/**
	 * 创建view
	 *
	 * 用于显示白板，view窗口与父窗口并不是实际的父子关系，view窗口始终能保持在父窗口的上层，当父窗口的位置或者大小发生变更时，需调用zego_whiteboard_show_view调整白板的位置和大小。
	 *
	 * @param parent 父窗口句柄
	 *
	 * @return 请求序号
	 */
	ZEGOAPI ZegoSeq ZEGOCALL zego_whiteboard_create_view(WINDOW_HANDLE parent);

	/**
	 * 注册创建view窗口回调
	 *
	 * @param fn 回调函数地址
	 * @param user_context 用户自定义数据
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_reg_create_view_callback(zego_whiteboard_create_view_callback fn, void* user_context);

	/**
	 * 显示或者隐藏view
	 *
	 * 1. 当父窗口移动、大小变更时，调用 zego_whiteboard_show_view(view_id, true) 调整白板位置
	 * 2. 当父窗口最小化、隐藏时，调用 zego_whiteboard_show_view(view_id, false) 隐藏白板
	 *
	 * @param view_id view标识
	 * @param show 显示或者隐藏
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_show_view(ZegoWhiteboardViewID view_id, bool show);

	/**
	 * 获取view的窗口句柄
	 *
	 * @param view_id view标识
	 *
	 * @return 窗口句柄
	 */
	ZEGOAPI WINDOW_HANDLE ZEGOCALL zego_whiteboard_get_view_hwnd(ZegoWhiteboardViewID view_id);

	/**
	 * 销毁view
	 *
	 * @param view_id view标识
	 *
	 * @return 请求序号
	 */
	ZEGOAPI ZegoSeq ZEGOCALL zego_whiteboard_destroy_view(ZegoWhiteboardViewID view_id);

	/**
	 * 注册销毁view回调
	 *
	 * @param fn 回调函数地址
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_reg_destroy_view_callback(zego_whiteboard_destroy_view_callback fn, void* user_context);

	/**
	 * 创建白板
	 *
	 * @param view_model 1. 如果创建纯白板，则必须设置的值有mode、name、aspect_widht、aspect_height、page_count
	 *                   2. 如果是文件白板，则必须设置的值有mode、file_info.file_id，其他均为SDK内部填充
	 *
	 * @return 请求序号
	 */
	ZEGOAPI ZegoSeq ZEGOCALL zego_whiteboard_create_whiteboard(struct ZegoWhiteboardViewModel view_model);

	/**
	 * 注册创建白板回调
	 *
	 * @param fn 回调函数地址
	 * @param user_context 用户自定义数据
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_reg_create_whiteboard_callback(zego_whiteboard_create_whiteboard_callback fn, void* user_context);

	/**
	 * 销毁白板
	 *
	 * @param whiteboard_id 白板标识
	 *
	 * @return 请求序号
	 */
	ZEGOAPI ZegoSeq ZEGOCALL zego_whiteboard_destroy_whiteboard(ZegoWhiteboardID whiteboard_id);

	/**
	 * 注册销毁白板回调
	 *
	 * @param user_context 用户自定义数据
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_reg_destroy_whiteboard_callback(zego_whiteboard_destroy_whiteboard_callback fn, void* user_context);

	/**
	 * 获取白板属性
	 *
	 * @param whiteboard_id 白板标识
	 *
	 * @return 白板属性
	 */
	ZEGOAPI struct ZegoWhiteboardViewModel* ZEGOCALL zego_whiteboard_get_whiteboard_view_model(ZegoWhiteboardID whiteboard_id);

	/**
	 * 设置view当前显示的白板
	 *
	 * @param view_id view标识
	 * @param whiteboard_id 白板标识
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_set_current_whiteboard(ZegoWhiteboardViewID view_id, ZegoWhiteboardID whiteboard_id);

	/**
	 * 获取view当前显示的白板
	 *
	 * @param view_id view标识
	 *
	 * @return 白板标识
	 */
	ZEGOAPI ZegoWhiteboardID ZEGOCALL zego_whiteboard_get_current_whiteboard(ZegoWhiteboardViewID view_id);

	/**
	 * 获取当前滚动百分比
	 *
	 * @param whiteboard_id 白板标识
	 * @param horizontal 水平滚动百分比
	 * @param vertical 垂直滚动百分比
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_get_current_scroll_percent(ZegoWhiteboardID whiteboard_id, float* horizontal_percent, float* vertical_percent);

	/**
	 * 滚动白板
	 *
	 * @param whiteboard_id 白板标识
	 * @param horizontal 水平滚动百分比（0.0000000~1.0000000）
	 * @param vertical 垂直滚动百分比（0.0000000~1.0000000）
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_scroll(ZegoWhiteboardID whiteboard_id, float horizontal_percent, float vertical_percent);

	/**
	 * 设置白板缩放系数
	 *
	 * @param whiteboard_id 白板标识
	 * @param factor 缩放系数（1.0000000~3.0000000）
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_set_scale_factor(ZegoWhiteboardID whiteboard_id, float factor);

	/**
	 * 获取白板缩放系数
	 *
	 * @param whiteboard_id 白板标识
	 *
	 * @return 白板缩放系数
	 */
	ZEGOAPI float ZEGOCALL zego_whiteboard_get_scale_factor(ZegoWhiteboardID whiteboard_id);


	/**
	 * 获取白板总页数
	 *
	 * 1. 当ZegoWhiteboardAttribute.mode是kZegoWhiteboardRenderModeWhiteboard时，获取的是创建白板时设置的总页数ZegoWhiteboardAttribute.page_count
	 * 2. 当ZegoWhiteboardAttribute.mode是kZegoWhiteboardRenderModeWhiteboardDocs时，获取的是文件的页数，txt与图片类型的文件页数固定为1
	 *
	 * @param whiteboard_id 白板标识
	 *
	 * @return 白板总页数
	 */
	ZEGOAPI unsigned int ZEGOCALL zego_whiteboard_get_page_count(ZegoWhiteboardID whiteboard_id);

	/**
	 * 获取白板当前显示的页码
	 *
	 * @param whiteboard_id 白板标识
	 *
	 * @return 当前页码
	 */
	ZEGOAPI unsigned int ZEGOCALL zego_whiteboard_get_current_page(ZegoWhiteboardID whiteboard_id);

	/**
	 * 下一页
	 *
	 * @param whiteboard_id 白板标识
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_next_page(ZegoWhiteboardID whiteboard_id);

	/**
	 * 上一页
	 *
	 * @param whiteboard_id 白板标识
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_previous_page(ZegoWhiteboardID whiteboard_id);

	/**
	 * 跳转到指定页
	 *
	 * @param whiteboard_id 白板标识
	 * @param page 指定页
	 */
	ZEGOAPI void ZEGOCALL  zego_whiteboard_flip_page(ZegoWhiteboardID whiteboard_id, unsigned int page);

	/**
	 * 动态ppt下一步
	 *
	 * 只当文件时动态ppt时才生效
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_docs_next_step(ZegoWhiteboardID whiteboard_id);

	/**
	 * 动态ppt上一步
	 *
	 * 只当文件时动态ppt时才生效
	 *
	 * @param whiteboard_id  白板标识
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_docs_previous_step(ZegoWhiteboardID whiteboard_id);

	/**
	 * 动态ppt获取当前页的总步数
	 *
	 * 只当文件时动态ppt时才生效
	 *
	 * @param whiteboard_id 白板标识
	 *
	 * @return 当前页总步数
	 */
	ZEGOAPI unsigned int ZEGOCALL zego_whiteboard_docs_get_step_count(ZegoWhiteboardID whiteboard_id);

	/**
	 * 获取动态ppt当前页的当前步数
	 *
	 * 只当文件时动态ppt时才生效
	 *
	 * @param whiteboard_id 白板标识
	 *
	 * @return 当前页的当前步数
	 */
	ZEGOAPI unsigned int ZEGOCALL zego_whiteboard_docs_get_current_step(ZegoWhiteboardID whiteboard_id);

	/**
	 * 注册房间内其他成员创建白板通知
	 *
	 * @param fn 回调函数地址
	 * @param user_context 用户自定义数据
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_reg_added_whiteboard_notify(zego_whiteboard_added_whiteboard_notify fn, void* user_context);

	/**
	 * 注册房间内其他成员销毁白板通知
	 *
	 * @param fn 回调函数地址
	 * @param user_context 用户自定义数据
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_reg_removed_whiteboard_notify(zego_whiteboard_removed_whiteboard_notify fn, void* user_context);

	/**
	 * 注册白板滚动变更通知
	 *
	 * @param fn 回调函数地址
	 * @param user_context 用户自定义数据
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_reg_scroll_changed_notify(zego_whiteboard_scroll_changed_notify fn, void* user_context);

	/**
	 * 注册动态ppt步数变更通知
	 *
	 * @param fn 回调函数地址
	 * @param user_context 用户自定义数据
	 */
	ZEGOAPI void ZEGOCALL zego_whiteboard_docs_reg_step_changed_notify(zego_whiteboard_docs_step_changed_notify fn, void* user_context);

#ifdef __cplusplus
}
#endif

#endif
